#include "uniModelExample_ecore/impl/FacultyImpl.hpp"

#ifdef NDEBUG
	#define DEBUG_MESSAGE(a) /**/
#else
	#define DEBUG_MESSAGE(a) a
#endif

#ifdef ACTIVITY_DEBUG_ON
    #define ACT_DEBUG(a) a
#else
    #define ACT_DEBUG(a) /**/
#endif

//#include "util/ProfileCallCount.hpp"

#include <cassert>
#include <iostream>
#include <sstream>

#include "abstractDataTypes/Bag.hpp"

#include "abstractDataTypes/SubsetUnion.hpp"
#include "ecore/EAnnotation.hpp"
#include "ecore/EClass.hpp"
#include "uniModelExample_ecore/impl/UniModelExample_ecorePackageImpl.hpp"

//Forward declaration includes
#include "persistence/interfaces/XLoadHandler.hpp" // used for Persistence
#include "persistence/interfaces/XSaveHandler.hpp" // used for Persistence
#include "uniModelExample_ecore/UniModelExample_ecoreFactory.hpp"
#include "uniModelExample_ecore/UniModelExample_ecorePackage.hpp"
#include <exception> // used in Persistence

#include "uniModelExample_ecore/Lecture.hpp"

#include "uniModelExample_ecore/NamedClass.hpp"

#include "uniModelExample_ecore/Student.hpp"

#include "uniModelExample_ecore/University.hpp"

#include "ecore/EcorePackage.hpp"
#include "ecore/EcoreFactory.hpp"
#include "uniModelExample_ecore/UniModelExample_ecorePackage.hpp"
#include "uniModelExample_ecore/UniModelExample_ecoreFactory.hpp"
#include "ecore/EAttribute.hpp"
#include "ecore/EStructuralFeature.hpp"

using namespace uniModelExample_ecore;

//*********************************
// Constructor / Destructor
//*********************************
FacultyImpl::FacultyImpl()
{
	//*********************************
	// Attribute Members
	//*********************************

	//*********************************
	// Reference Members
	//*********************************
	//References
		m_lectures.reset(new Bag<uniModelExample_ecore::Lecture>());
	
	

		m_students.reset(new Bag<uniModelExample_ecore::Student>());
	
	

	

	//Init references
	
	

	
	

	
}

FacultyImpl::~FacultyImpl()
{
#ifdef SHOW_DELETION
	std::cout << "-------------------------------------------------------------------------------------------------\r\ndelete Faculty "<< this << "\r\n------------------------------------------------------------------------ " << std::endl;
#endif
}


//Additional constructor for the containments back reference
			FacultyImpl::FacultyImpl(std::weak_ptr<uniModelExample_ecore::University > par_university)
			:FacultyImpl()
			{
			    m_university = par_university;
			}






FacultyImpl::FacultyImpl(const FacultyImpl & obj):FacultyImpl()
{
	//create copy of all Attributes
	#ifdef SHOW_COPIES
	std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\r\ncopy Faculty "<< this << "\r\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ " << std::endl;
	#endif
	m_name = obj.getName();

	//copy references with no containment (soft copy)
	
	std::shared_ptr<Bag<uniModelExample_ecore::Student>> _students = obj.getStudents();
	m_students.reset(new Bag<uniModelExample_ecore::Student>(*(obj.getStudents().get())));

	m_university  = obj.getUniversity();


	//Clone references with containment (deep copy)

	std::shared_ptr<Bag<uniModelExample_ecore::Lecture>> _lecturesList = obj.getLectures();
	for(std::shared_ptr<uniModelExample_ecore::Lecture> _lectures : *_lecturesList)
	{
		this->getLectures()->add(std::shared_ptr<uniModelExample_ecore::Lecture>(std::dynamic_pointer_cast<uniModelExample_ecore::Lecture>(_lectures->copy())));
	}
	#ifdef SHOW_SUBSET_UNION
		std::cout << "Copying the Subset: " << "m_lectures" << std::endl;
	#endif

	
	
}

std::shared_ptr<ecore::EObject>  FacultyImpl::copy() const
{
	std::shared_ptr<FacultyImpl> element(new FacultyImpl(*this));
	element->setThisFacultyPtr(element);
	return element;
}

std::shared_ptr<ecore::EClass> FacultyImpl::eStaticClass() const
{
	return UniModelExample_ecorePackageImpl::eInstance()->getFaculty_EClass();
}

//*********************************
// Attribute Setter Getter
//*********************************

//*********************************
// Operations
//*********************************

//*********************************
// References
//*********************************
std::shared_ptr<Bag<uniModelExample_ecore::Lecture>> FacultyImpl::getLectures() const
{

    return m_lectures;
}


std::shared_ptr<Bag<uniModelExample_ecore::Student>> FacultyImpl::getStudents() const
{

    return m_students;
}


std::weak_ptr<uniModelExample_ecore::University > FacultyImpl::getUniversity() const
{

    return m_university;
}
void FacultyImpl::setUniversity(std::shared_ptr<uniModelExample_ecore::University> _university)
{
    m_university = _university;
}

//*********************************
// Union Getter
//*********************************


std::shared_ptr<Faculty> FacultyImpl::getThisFacultyPtr() const
{
	return m_thisFacultyPtr.lock();
}
void FacultyImpl::setThisFacultyPtr(std::weak_ptr<Faculty> thisFacultyPtr)
{
	m_thisFacultyPtr = thisFacultyPtr;
	setThisNamedClassPtr(thisFacultyPtr);
}
std::shared_ptr<ecore::EObject> FacultyImpl::eContainer() const
{
	if(auto wp = m_university.lock())
	{
		return wp;
	}
	return nullptr;
}

//*********************************
// Structural Feature Getter/Setter
//*********************************
Any FacultyImpl::eGet(int featureID, bool resolve, bool coreType) const
{
	switch(featureID)
	{
		case UniModelExample_ecorePackage::FACULTY_EREFERENCE_LECTURES:
			return eAny(getLectures()); //12
		case UniModelExample_ecorePackage::FACULTY_EREFERENCE_STUDENTS:
			return eAny(getStudents()); //11
		case UniModelExample_ecorePackage::FACULTY_EREFERENCE_UNIVERSITY:
			return eAny(getUniversity()); //13
	}
	return NamedClassImpl::eGet(featureID, resolve, coreType);
}
bool FacultyImpl::internalEIsSet(int featureID) const
{
	switch(featureID)
	{
		case UniModelExample_ecorePackage::FACULTY_EREFERENCE_LECTURES:
			return getLectures() != nullptr; //12
		case UniModelExample_ecorePackage::FACULTY_EREFERENCE_STUDENTS:
			return getStudents() != nullptr; //11
		case UniModelExample_ecorePackage::FACULTY_EREFERENCE_UNIVERSITY:
			return getUniversity().lock() != nullptr; //13
	}
	return NamedClassImpl::internalEIsSet(featureID);
}
bool FacultyImpl::eSet(int featureID, Any newValue)
{
	switch(featureID)
	{
		case UniModelExample_ecorePackage::FACULTY_EREFERENCE_UNIVERSITY:
		{
			// BOOST CAST
			std::shared_ptr<uniModelExample_ecore::University> _university = newValue->get<std::shared_ptr<uniModelExample_ecore::University>>();
			setUniversity(_university); //13
			return true;
		}
	}

	return NamedClassImpl::eSet(featureID, newValue);
}

//*********************************
// Persistence Functions
//*********************************
void FacultyImpl::load(std::shared_ptr<persistence::interfaces::XLoadHandler> loadHandler)
{
	std::map<std::string, std::string> attr_list = loadHandler->getAttributeList();
	loadAttributes(loadHandler, attr_list);

	//
	// Create new objects (from references (containment == true))
	//
	// get UniModelExample_ecoreFactory
	std::shared_ptr<uniModelExample_ecore::UniModelExample_ecoreFactory> modelFactory = uniModelExample_ecore::UniModelExample_ecoreFactory::eInstance();
	int numNodes = loadHandler->getNumOfChildNodes();
	for(int ii = 0; ii < numNodes; ii++)
	{
		loadNode(loadHandler->getNextNodeName(), loadHandler, modelFactory);
	}
}		

void FacultyImpl::loadAttributes(std::shared_ptr<persistence::interfaces::XLoadHandler> loadHandler, std::map<std::string, std::string> attr_list)
{
	try
	{
		std::map<std::string, std::string>::const_iterator iter;
		std::shared_ptr<ecore::EClass> metaClass = this->eClass(); // get MetaClass
		iter = attr_list.find("students");
		if ( iter != attr_list.end() )
		{
			// add unresolvedReference to loadHandler's list
			loadHandler->addUnresolvedReference(iter->second, loadHandler->getCurrentObject(), metaClass->getEStructuralFeature("students")); // TODO use getEStructuralFeature() with id, for faster access to EStructuralFeature
		}
	}
	catch (std::exception& e)
	{
		std::cout << "| ERROR    | " << e.what() << std::endl;
	}
	catch (...) 
	{
		std::cout << "| ERROR    | " <<  "Exception occurred" << std::endl;
	}

	NamedClassImpl::loadAttributes(loadHandler, attr_list);
}

void FacultyImpl::loadNode(std::string nodeName, std::shared_ptr<persistence::interfaces::XLoadHandler> loadHandler, std::shared_ptr<uniModelExample_ecore::UniModelExample_ecoreFactory> modelFactory)
{

	try
	{
		if ( nodeName.compare("lectures") == 0 )
		{
  			std::string typeName = loadHandler->getCurrentXSITypeName();
			if (typeName.empty())
			{
				typeName = "Lecture";
			}
			std::shared_ptr<uniModelExample_ecore::Lecture> lectures = std::dynamic_pointer_cast<uniModelExample_ecore::Lecture>(modelFactory->create(typeName));
			if (lectures != nullptr)
			{
				std::shared_ptr<Bag<uniModelExample_ecore::Lecture>> list_lectures = this->getLectures();
				list_lectures->push_back(lectures);
				loadHandler->handleChild(lectures);
			}
			return;
		}
	}
	catch (std::exception& e)
	{
		std::cout << "| ERROR    | " << e.what() << std::endl;
	}
	catch (...) 
	{
		std::cout << "| ERROR    | " <<  "Exception occurred" << std::endl;
	}

	NamedClassImpl::loadNode(nodeName, loadHandler, modelFactory);
}

void FacultyImpl::resolveReferences(const int featureID, std::list<std::shared_ptr<ecore::EObject> > references)
{
	switch(featureID)
	{
		case UniModelExample_ecorePackage::FACULTY_EREFERENCE_STUDENTS:
		{
			std::shared_ptr<Bag<uniModelExample_ecore::Student>> _students = getStudents();
			for(std::shared_ptr<ecore::EObject> ref : references)
			{
				std::shared_ptr<uniModelExample_ecore::Student> _r = std::dynamic_pointer_cast<uniModelExample_ecore::Student>(ref);
				if (_r != nullptr)
				{
					_students->push_back(_r);
				}				
			}
			return;
		}

		case UniModelExample_ecorePackage::FACULTY_EREFERENCE_UNIVERSITY:
		{
			if (references.size() == 1)
			{
				// Cast object to correct type
				std::shared_ptr<uniModelExample_ecore::University> _university = std::dynamic_pointer_cast<uniModelExample_ecore::University>( references.front() );
				setUniversity(_university);
			}
			
			return;
		}
	}
	NamedClassImpl::resolveReferences(featureID, references);
}

void FacultyImpl::save(std::shared_ptr<persistence::interfaces::XSaveHandler> saveHandler) const
{
	saveContent(saveHandler);

	NamedClassImpl::saveContent(saveHandler);
	
	ecore::EObjectImpl::saveContent(saveHandler);
	
}

void FacultyImpl::saveContent(std::shared_ptr<persistence::interfaces::XSaveHandler> saveHandler) const
{
	try
	{
		std::shared_ptr<uniModelExample_ecore::UniModelExample_ecorePackage> package = uniModelExample_ecore::UniModelExample_ecorePackage::eInstance();

	

		// Add references
		std::shared_ptr<Bag<uniModelExample_ecore::Student>> students_list = this->getStudents();
		for (std::shared_ptr<uniModelExample_ecore::Student > object : *students_list)
		{ 
			saveHandler->addReferences("students", object);
		}


		//
		// Add new tags (from references)
		//
		std::shared_ptr<ecore::EClass> metaClass = this->eClass();
		// Save 'lectures'
		std::shared_ptr<Bag<uniModelExample_ecore::Lecture>> list_lectures = this->getLectures();
		for (std::shared_ptr<uniModelExample_ecore::Lecture> lectures : *list_lectures) 
		{
			saveHandler->addReference(lectures, "lectures", lectures->eClass() != package->getLecture_EClass());
		}
	}
	catch (std::exception& e)
	{
		std::cout << "| ERROR    | " << e.what() << std::endl;
	}
}

