#include "uniModelExample_ecore/impl/NamedClassImpl.hpp"

#ifdef NDEBUG
	#define DEBUG_MESSAGE(a) /**/
#else
	#define DEBUG_MESSAGE(a) a
#endif

#ifdef ACTIVITY_DEBUG_ON
    #define ACT_DEBUG(a) a
#else
    #define ACT_DEBUG(a) /**/
#endif

//#include "util/ProfileCallCount.hpp"

#include <cassert>
#include <iostream>
#include <sstream>


#include "abstractDataTypes/SubsetUnion.hpp"
#include "ecore/EAnnotation.hpp"
#include "ecore/EClass.hpp"
#include "uniModelExample_ecore/impl/UniModelExample_ecorePackageImpl.hpp"

//Forward declaration includes
#include "persistence/interfaces/XLoadHandler.hpp" // used for Persistence
#include "persistence/interfaces/XSaveHandler.hpp" // used for Persistence
#include "uniModelExample_ecore/UniModelExample_ecoreFactory.hpp"
#include "uniModelExample_ecore/UniModelExample_ecorePackage.hpp"
#include <exception> // used in Persistence


#include "ecore/EcorePackage.hpp"
#include "ecore/EcoreFactory.hpp"
#include "uniModelExample_ecore/UniModelExample_ecorePackage.hpp"
#include "uniModelExample_ecore/UniModelExample_ecoreFactory.hpp"
#include "ecore/EAttribute.hpp"
#include "ecore/EStructuralFeature.hpp"

using namespace uniModelExample_ecore;

//*********************************
// Constructor / Destructor
//*********************************
NamedClassImpl::NamedClassImpl()
{
	//*********************************
	// Attribute Members
	//*********************************
	
	//*********************************
	// Reference Members
	//*********************************
	//References

	//Init references
}

NamedClassImpl::~NamedClassImpl()
{
#ifdef SHOW_DELETION
	std::cout << "-------------------------------------------------------------------------------------------------\r\ndelete NamedClass "<< this << "\r\n------------------------------------------------------------------------ " << std::endl;
#endif
}




NamedClassImpl::NamedClassImpl(const NamedClassImpl & obj):NamedClassImpl()
{
	//create copy of all Attributes
	#ifdef SHOW_COPIES
	std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\r\ncopy NamedClass "<< this << "\r\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ " << std::endl;
	#endif
	m_name = obj.getName();

	//copy references with no containment (soft copy)
	

	//Clone references with containment (deep copy)


}

std::shared_ptr<ecore::EObject>  NamedClassImpl::copy() const
{
	std::shared_ptr<NamedClassImpl> element(new NamedClassImpl(*this));
	element->setThisNamedClassPtr(element);
	return element;
}

std::shared_ptr<ecore::EClass> NamedClassImpl::eStaticClass() const
{
	return UniModelExample_ecorePackageImpl::eInstance()->getNamedClass_EClass();
}

//*********************************
// Attribute Setter Getter
//*********************************
void NamedClassImpl::setName(std::string _name)
{
	m_name = _name;
} 

std::string NamedClassImpl::getName() const 
{
	return m_name;
}

//*********************************
// Operations
//*********************************

//*********************************
// References
//*********************************

//*********************************
// Union Getter
//*********************************


std::shared_ptr<NamedClass> NamedClassImpl::getThisNamedClassPtr() const
{
	return m_thisNamedClassPtr.lock();
}
void NamedClassImpl::setThisNamedClassPtr(std::weak_ptr<NamedClass> thisNamedClassPtr)
{
	m_thisNamedClassPtr = thisNamedClassPtr;
}
std::shared_ptr<ecore::EObject> NamedClassImpl::eContainer() const
{
	return nullptr;
}

//*********************************
// Structural Feature Getter/Setter
//*********************************
Any NamedClassImpl::eGet(int featureID, bool resolve, bool coreType) const
{
	switch(featureID)
	{
		case UniModelExample_ecorePackage::NAMEDCLASS_EATTRIBUTE_NAME:
			return eAny(getName()); //50
	}
	return ecore::EObjectImpl::eGet(featureID, resolve, coreType);
}
bool NamedClassImpl::internalEIsSet(int featureID) const
{
	switch(featureID)
	{
		case UniModelExample_ecorePackage::NAMEDCLASS_EATTRIBUTE_NAME:
			return getName() != ""; //50
	}
	return ecore::EObjectImpl::internalEIsSet(featureID);
}
bool NamedClassImpl::eSet(int featureID, Any newValue)
{
	switch(featureID)
	{
		case UniModelExample_ecorePackage::NAMEDCLASS_EATTRIBUTE_NAME:
		{
			// BOOST CAST
			std::string _name = newValue->get<std::string>();
			setName(_name); //50
			return true;
		}
	}

	return ecore::EObjectImpl::eSet(featureID, newValue);
}

//*********************************
// Persistence Functions
//*********************************
void NamedClassImpl::load(std::shared_ptr<persistence::interfaces::XLoadHandler> loadHandler)
{
	std::map<std::string, std::string> attr_list = loadHandler->getAttributeList();
	loadAttributes(loadHandler, attr_list);

	//
	// Create new objects (from references (containment == true))
	//
	// get UniModelExample_ecoreFactory
	std::shared_ptr<uniModelExample_ecore::UniModelExample_ecoreFactory> modelFactory = uniModelExample_ecore::UniModelExample_ecoreFactory::eInstance();
	int numNodes = loadHandler->getNumOfChildNodes();
	for(int ii = 0; ii < numNodes; ii++)
	{
		loadNode(loadHandler->getNextNodeName(), loadHandler, modelFactory);
	}
}		

void NamedClassImpl::loadAttributes(std::shared_ptr<persistence::interfaces::XLoadHandler> loadHandler, std::map<std::string, std::string> attr_list)
{
	try
	{
		std::map<std::string, std::string>::const_iterator iter;
	
		iter = attr_list.find("name");
		if ( iter != attr_list.end() )
		{
			// this attribute is a 'std::string'
			std::string value;
			value = iter->second;
			this->setName(value);
		}
	}
	catch (std::exception& e)
	{
		std::cout << "| ERROR    | " << e.what() << std::endl;
	}
	catch (...) 
	{
		std::cout << "| ERROR    | " <<  "Exception occurred" << std::endl;
	}

	ecore::EObjectImpl::loadAttributes(loadHandler, attr_list);
}

void NamedClassImpl::loadNode(std::string nodeName, std::shared_ptr<persistence::interfaces::XLoadHandler> loadHandler, std::shared_ptr<uniModelExample_ecore::UniModelExample_ecoreFactory> modelFactory)
{


	ecore::EObjectImpl::loadNode(nodeName, loadHandler, ecore::EcoreFactory::eInstance());
}

void NamedClassImpl::resolveReferences(const int featureID, std::list<std::shared_ptr<ecore::EObject> > references)
{
	ecore::EObjectImpl::resolveReferences(featureID, references);
}

void NamedClassImpl::save(std::shared_ptr<persistence::interfaces::XSaveHandler> saveHandler) const
{
	saveContent(saveHandler);

	
	ecore::EObjectImpl::saveContent(saveHandler);
	
}

void NamedClassImpl::saveContent(std::shared_ptr<persistence::interfaces::XSaveHandler> saveHandler) const
{
	try
	{
		std::shared_ptr<uniModelExample_ecore::UniModelExample_ecorePackage> package = uniModelExample_ecore::UniModelExample_ecorePackage::eInstance();

	
 
		// Add attributes
		if ( this->eIsSet(package->getNamedClass_EAttribute_name()) )
		{
			saveHandler->addAttribute("name", this->getName());
		}

	}
	catch (std::exception& e)
	{
		std::cout << "| ERROR    | " << e.what() << std::endl;
	}
}

