#include "uniModelExample_ecore/impl/StudentImpl.hpp"

#ifdef NDEBUG
	#define DEBUG_MESSAGE(a) /**/
#else
	#define DEBUG_MESSAGE(a) a
#endif

#ifdef ACTIVITY_DEBUG_ON
    #define ACT_DEBUG(a) a
#else
    #define ACT_DEBUG(a) /**/
#endif

//#include "util/ProfileCallCount.hpp"

#include <cassert>
#include <iostream>
#include <sstream>

#include "abstractDataTypes/Bag.hpp"

#include "abstractDataTypes/SubsetUnion.hpp"
#include "ecore/EAnnotation.hpp"
#include "ecore/EClass.hpp"
#include "uniModelExample_ecore/impl/UniModelExample_ecorePackageImpl.hpp"

//Forward declaration includes
#include "persistence/interfaces/XLoadHandler.hpp" // used for Persistence
#include "persistence/interfaces/XSaveHandler.hpp" // used for Persistence
#include "uniModelExample_ecore/UniModelExample_ecoreFactory.hpp"
#include "uniModelExample_ecore/UniModelExample_ecorePackage.hpp"
#include <exception> // used in Persistence

#include "uniModelExample_ecore/Exam.hpp"

#include "uniModelExample_ecore/Lecture.hpp"

#include "uniModelExample_ecore/NamedClass.hpp"

#include "ecore/EcorePackage.hpp"
#include "ecore/EcoreFactory.hpp"
#include "uniModelExample_ecore/UniModelExample_ecorePackage.hpp"
#include "uniModelExample_ecore/UniModelExample_ecoreFactory.hpp"
#include "ecore/EAttribute.hpp"
#include "ecore/EStructuralFeature.hpp"

using namespace uniModelExample_ecore;

//*********************************
// Constructor / Destructor
//*********************************
StudentImpl::StudentImpl()
{
	//*********************************
	// Attribute Members
	//*********************************
	
	//*********************************
	// Reference Members
	//*********************************
	//References
		m_exams.reset(new Bag<uniModelExample_ecore::Exam>());
	
	

		m_lectures.reset(new Bag<uniModelExample_ecore::Lecture>());
	
	

	//Init references
	
	

	
	
}

StudentImpl::~StudentImpl()
{
#ifdef SHOW_DELETION
	std::cout << "-------------------------------------------------------------------------------------------------\r\ndelete Student "<< this << "\r\n------------------------------------------------------------------------ " << std::endl;
#endif
}




StudentImpl::StudentImpl(const StudentImpl & obj):StudentImpl()
{
	//create copy of all Attributes
	#ifdef SHOW_COPIES
	std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\r\ncopy Student "<< this << "\r\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ " << std::endl;
	#endif
	m_matNum = obj.getMatNum();
	m_name = obj.getName();

	//copy references with no containment (soft copy)
	
	std::shared_ptr<Bag<uniModelExample_ecore::Lecture>> _lectures = obj.getLectures();
	m_lectures.reset(new Bag<uniModelExample_ecore::Lecture>(*(obj.getLectures().get())));


	//Clone references with containment (deep copy)

	std::shared_ptr<Bag<uniModelExample_ecore::Exam>> _examsList = obj.getExams();
	for(std::shared_ptr<uniModelExample_ecore::Exam> _exams : *_examsList)
	{
		this->getExams()->add(std::shared_ptr<uniModelExample_ecore::Exam>(std::dynamic_pointer_cast<uniModelExample_ecore::Exam>(_exams->copy())));
	}
	#ifdef SHOW_SUBSET_UNION
		std::cout << "Copying the Subset: " << "m_exams" << std::endl;
	#endif

	
	
}

std::shared_ptr<ecore::EObject>  StudentImpl::copy() const
{
	std::shared_ptr<StudentImpl> element(new StudentImpl(*this));
	element->setThisStudentPtr(element);
	return element;
}

std::shared_ptr<ecore::EClass> StudentImpl::eStaticClass() const
{
	return UniModelExample_ecorePackageImpl::eInstance()->getStudent_EClass();
}

//*********************************
// Attribute Setter Getter
//*********************************
void StudentImpl::setMatNum(int _matNum)
{
	m_matNum = _matNum;
} 

int StudentImpl::getMatNum() const 
{
	return m_matNum;
}

//*********************************
// Operations
//*********************************
float StudentImpl::getAvgGrade()
{
	//ADD_COUNT(__PRETTY_FUNCTION__)
	//generated from body annotation
		float avgGrade = 0;

//In MDE4CPP Bag<> is used as container class
//Get container of all Exams of student
	std::shared_ptr<Bag<uniModelExample_ecore::Exam>> exams = this->getExams();

//If there are no Exams for this student, return 0
	if(exams->size() == 0){return avgGrade;}

//Loop over all Exam objects
//As we don’t need to modify the container, we use const_iterator
	for(Bag<uniModelExample_ecore::Exam>::const_iterator it = exams->begin(); it!=exams->end(); it++)
    		{
       		avgGrade += (*it)->getScore(); 
    		}

	avgGrade /= getExams()->size();

	return avgGrade;


	//end of body
}

//*********************************
// References
//*********************************
std::shared_ptr<Bag<uniModelExample_ecore::Exam>> StudentImpl::getExams() const
{

    return m_exams;
}


std::shared_ptr<Bag<uniModelExample_ecore::Lecture>> StudentImpl::getLectures() const
{

    return m_lectures;
}


//*********************************
// Union Getter
//*********************************


std::shared_ptr<Student> StudentImpl::getThisStudentPtr() const
{
	return m_thisStudentPtr.lock();
}
void StudentImpl::setThisStudentPtr(std::weak_ptr<Student> thisStudentPtr)
{
	m_thisStudentPtr = thisStudentPtr;
	setThisNamedClassPtr(thisStudentPtr);
}
std::shared_ptr<ecore::EObject> StudentImpl::eContainer() const
{
	return nullptr;
}

//*********************************
// Structural Feature Getter/Setter
//*********************************
Any StudentImpl::eGet(int featureID, bool resolve, bool coreType) const
{
	switch(featureID)
	{
		case UniModelExample_ecorePackage::STUDENT_EREFERENCE_EXAMS:
			return eAny(getExams()); //33
		case UniModelExample_ecorePackage::STUDENT_EREFERENCE_LECTURES:
			return eAny(getLectures()); //32
		case UniModelExample_ecorePackage::STUDENT_EATTRIBUTE_MATNUM:
			return eAny(getMatNum()); //31
	}
	return NamedClassImpl::eGet(featureID, resolve, coreType);
}
bool StudentImpl::internalEIsSet(int featureID) const
{
	switch(featureID)
	{
		case UniModelExample_ecorePackage::STUDENT_EREFERENCE_EXAMS:
			return getExams() != nullptr; //33
		case UniModelExample_ecorePackage::STUDENT_EREFERENCE_LECTURES:
			return getLectures() != nullptr; //32
		case UniModelExample_ecorePackage::STUDENT_EATTRIBUTE_MATNUM:
			return getMatNum() != 0; //31
	}
	return NamedClassImpl::internalEIsSet(featureID);
}
bool StudentImpl::eSet(int featureID, Any newValue)
{
	switch(featureID)
	{
		case UniModelExample_ecorePackage::STUDENT_EATTRIBUTE_MATNUM:
		{
			// BOOST CAST
			int _matNum = newValue->get<int>();
			setMatNum(_matNum); //31
			return true;
		}
	}

	return NamedClassImpl::eSet(featureID, newValue);
}

//*********************************
// Persistence Functions
//*********************************
void StudentImpl::load(std::shared_ptr<persistence::interfaces::XLoadHandler> loadHandler)
{
	std::map<std::string, std::string> attr_list = loadHandler->getAttributeList();
	loadAttributes(loadHandler, attr_list);

	//
	// Create new objects (from references (containment == true))
	//
	// get UniModelExample_ecoreFactory
	std::shared_ptr<uniModelExample_ecore::UniModelExample_ecoreFactory> modelFactory = uniModelExample_ecore::UniModelExample_ecoreFactory::eInstance();
	int numNodes = loadHandler->getNumOfChildNodes();
	for(int ii = 0; ii < numNodes; ii++)
	{
		loadNode(loadHandler->getNextNodeName(), loadHandler, modelFactory);
	}
}		

void StudentImpl::loadAttributes(std::shared_ptr<persistence::interfaces::XLoadHandler> loadHandler, std::map<std::string, std::string> attr_list)
{
	try
	{
		std::map<std::string, std::string>::const_iterator iter;
	
		iter = attr_list.find("matNum");
		if ( iter != attr_list.end() )
		{
			// this attribute is a 'int'
			int value;
			std::istringstream ( iter->second ) >> value;
			this->setMatNum(value);
		}
		std::shared_ptr<ecore::EClass> metaClass = this->eClass(); // get MetaClass
		iter = attr_list.find("lectures");
		if ( iter != attr_list.end() )
		{
			// add unresolvedReference to loadHandler's list
			loadHandler->addUnresolvedReference(iter->second, loadHandler->getCurrentObject(), metaClass->getEStructuralFeature("lectures")); // TODO use getEStructuralFeature() with id, for faster access to EStructuralFeature
		}
	}
	catch (std::exception& e)
	{
		std::cout << "| ERROR    | " << e.what() << std::endl;
	}
	catch (...) 
	{
		std::cout << "| ERROR    | " <<  "Exception occurred" << std::endl;
	}

	NamedClassImpl::loadAttributes(loadHandler, attr_list);
}

void StudentImpl::loadNode(std::string nodeName, std::shared_ptr<persistence::interfaces::XLoadHandler> loadHandler, std::shared_ptr<uniModelExample_ecore::UniModelExample_ecoreFactory> modelFactory)
{

	try
	{
		if ( nodeName.compare("exams") == 0 )
		{
  			std::string typeName = loadHandler->getCurrentXSITypeName();
			if (typeName.empty())
			{
				typeName = "Exam";
			}
			std::shared_ptr<uniModelExample_ecore::Exam> exams = std::dynamic_pointer_cast<uniModelExample_ecore::Exam>(modelFactory->create(typeName));
			if (exams != nullptr)
			{
				std::shared_ptr<Bag<uniModelExample_ecore::Exam>> list_exams = this->getExams();
				list_exams->push_back(exams);
				loadHandler->handleChild(exams);
			}
			return;
		}
	}
	catch (std::exception& e)
	{
		std::cout << "| ERROR    | " << e.what() << std::endl;
	}
	catch (...) 
	{
		std::cout << "| ERROR    | " <<  "Exception occurred" << std::endl;
	}

	NamedClassImpl::loadNode(nodeName, loadHandler, modelFactory);
}

void StudentImpl::resolveReferences(const int featureID, std::list<std::shared_ptr<ecore::EObject> > references)
{
	switch(featureID)
	{
		case UniModelExample_ecorePackage::STUDENT_EREFERENCE_LECTURES:
		{
			std::shared_ptr<Bag<uniModelExample_ecore::Lecture>> _lectures = getLectures();
			for(std::shared_ptr<ecore::EObject> ref : references)
			{
				std::shared_ptr<uniModelExample_ecore::Lecture> _r = std::dynamic_pointer_cast<uniModelExample_ecore::Lecture>(ref);
				if (_r != nullptr)
				{
					_lectures->push_back(_r);
				}				
			}
			return;
		}
	}
	NamedClassImpl::resolveReferences(featureID, references);
}

void StudentImpl::save(std::shared_ptr<persistence::interfaces::XSaveHandler> saveHandler) const
{
	saveContent(saveHandler);

	NamedClassImpl::saveContent(saveHandler);
	
	ecore::EObjectImpl::saveContent(saveHandler);
	
}

void StudentImpl::saveContent(std::shared_ptr<persistence::interfaces::XSaveHandler> saveHandler) const
{
	try
	{
		std::shared_ptr<uniModelExample_ecore::UniModelExample_ecorePackage> package = uniModelExample_ecore::UniModelExample_ecorePackage::eInstance();

	
 
		// Add attributes
		if ( this->eIsSet(package->getStudent_EAttribute_matNum()) )
		{
			saveHandler->addAttribute("matNum", this->getMatNum());
		}

		// Add references
		std::shared_ptr<Bag<uniModelExample_ecore::Lecture>> lectures_list = this->getLectures();
		for (std::shared_ptr<uniModelExample_ecore::Lecture > object : *lectures_list)
		{ 
			saveHandler->addReferences("lectures", object);
		}


		//
		// Add new tags (from references)
		//
		std::shared_ptr<ecore::EClass> metaClass = this->eClass();
		// Save 'exams'
		std::shared_ptr<Bag<uniModelExample_ecore::Exam>> list_exams = this->getExams();
		for (std::shared_ptr<uniModelExample_ecore::Exam> exams : *list_exams) 
		{
			saveHandler->addReference(exams, "exams", exams->eClass() != package->getExam_EClass());
		}
	}
	catch (std::exception& e)
	{
		std::cout << "| ERROR    | " << e.what() << std::endl;
	}
}

