#include "uniModelExample_ecore/impl/UniModelExample_ecoreFactoryImpl.hpp"

#include <cassert>

#include "abstractDataTypes/SubsetUnion.hpp"
#include "uniModelExample_ecore/UniModelExample_ecorePackage.hpp"

#include "ecore/EClass.hpp"


#include "uniModelExample_ecore/impl/ExamImpl.hpp"
#include "uniModelExample_ecore/impl/FacultyImpl.hpp"
#include "uniModelExample_ecore/impl/LectureImpl.hpp"
#include "uniModelExample_ecore/impl/NamedClassImpl.hpp"
#include "uniModelExample_ecore/impl/StudentImpl.hpp"
#include "uniModelExample_ecore/impl/UniversityImpl.hpp"

using namespace uniModelExample_ecore;

//*********************************
// Constructor / Destructor
//*********************************

UniModelExample_ecoreFactoryImpl::UniModelExample_ecoreFactoryImpl()
{
	m_idMap.insert(std::make_pair("Exam", UniModelExample_ecorePackage::EXAM_ECLASS));
	m_idMap.insert(std::make_pair("Faculty", UniModelExample_ecorePackage::FACULTY_ECLASS));
	m_idMap.insert(std::make_pair("Lecture", UniModelExample_ecorePackage::LECTURE_ECLASS));
	m_idMap.insert(std::make_pair("NamedClass", UniModelExample_ecorePackage::NAMEDCLASS_ECLASS));
	m_idMap.insert(std::make_pair("Student", UniModelExample_ecorePackage::STUDENT_ECLASS));
	m_idMap.insert(std::make_pair("University", UniModelExample_ecorePackage::UNIVERSITY_ECLASS));
}

UniModelExample_ecoreFactoryImpl::~UniModelExample_ecoreFactoryImpl()
{
}

UniModelExample_ecoreFactory* UniModelExample_ecoreFactoryImpl::create()
{
	return new UniModelExample_ecoreFactoryImpl();
}

//*********************************
// creators
//*********************************

std::shared_ptr<ecore::EObject> UniModelExample_ecoreFactoryImpl::create(const unsigned int classID,  std::shared_ptr<ecore::EObject> container /*= nullptr*/, const unsigned int referenceID/* = -1*/) const
{
	switch(classID)
	{
		case UniModelExample_ecorePackage::EXAM_ECLASS:
		{
				return this->createExam();
			
		}
		case UniModelExample_ecorePackage::FACULTY_ECLASS:
		{
			if (nullptr == container)
			{
				return this->createFaculty();
			}
			else
			{
				auto castedContainer = std::dynamic_pointer_cast<uniModelExample_ecore::University>(container);
				assert(castedContainer);
				return std::shared_ptr<Faculty>(this->createFaculty_in_University(castedContainer));
			}
		}
		case UniModelExample_ecorePackage::LECTURE_ECLASS:
		{
				return this->createLecture();
			
		}
		case UniModelExample_ecorePackage::NAMEDCLASS_ECLASS:
		{
				return this->createNamedClass();
			
		}
		case UniModelExample_ecorePackage::STUDENT_ECLASS:
		{
				return this->createStudent();
			
		}
		case UniModelExample_ecorePackage::UNIVERSITY_ECLASS:
		{
				return this->createUniversity();
			
		}
	default:
	   	    std::cerr << __PRETTY_FUNCTION__ << " ID " << classID <<" not found" << std::endl;
	}
	return nullptr;
}

std::shared_ptr<ecore::EObject> UniModelExample_ecoreFactoryImpl::create(std::shared_ptr<ecore::EClass> _class) const
{
	return create(_class, nullptr);
}

std::shared_ptr<ecore::EObject> UniModelExample_ecoreFactoryImpl::create(std::shared_ptr<ecore::EClass> _class, std::shared_ptr<EObject> _container) const
{
	if(_class->isAbstract())
    {
    	return nullptr;
   	}

	int _classID = _class->eClass()->getClassifierID();
	return create(_classID, _container);
}

std::shared_ptr<ecore::EObject> UniModelExample_ecoreFactoryImpl::create(std::string _className) const
{
	auto iter = m_idMap.find(_className);
	if(iter != m_idMap.end())
    {
		//get the ID
        unsigned int id = iter->second;
		return create(id);
    }

    return nullptr;
}

std::shared_ptr<ecore::EObject> UniModelExample_ecoreFactoryImpl::create(std::string _className, std::shared_ptr<EObject> _container, const unsigned int referenceID) const
{
	auto iter = m_idMap.find(_className);
	if(iter != m_idMap.end())
    {
		//get the ID
        unsigned int id = iter->second;
		return create(id, _container, referenceID);
    }

    return nullptr;
}


std::shared_ptr<Exam> UniModelExample_ecoreFactoryImpl::createExam() const
{
	std::shared_ptr<ExamImpl> element(new ExamImpl());
	element->setThisExamPtr(element);
	return element;
}
std::shared_ptr<Faculty> UniModelExample_ecoreFactoryImpl::createFaculty() const
{
	std::shared_ptr<FacultyImpl> element(new FacultyImpl());
	element->setThisFacultyPtr(element);
	return element;
}
std::shared_ptr<Faculty> UniModelExample_ecoreFactoryImpl::createFaculty_in_University(std::weak_ptr<uniModelExample_ecore::University > par_university) const
{
	std::shared_ptr<FacultyImpl> element(new FacultyImpl(par_university));
	if(auto wp = par_university.lock())
	{
			wp->getFaculties()->push_back(element);
	}
	element->setThisFacultyPtr(element);
	return element;
	
}

std::shared_ptr<Lecture> UniModelExample_ecoreFactoryImpl::createLecture() const
{
	std::shared_ptr<LectureImpl> element(new LectureImpl());
	element->setThisLecturePtr(element);
	return element;
}
std::shared_ptr<NamedClass> UniModelExample_ecoreFactoryImpl::createNamedClass() const
{
	std::shared_ptr<NamedClassImpl> element(new NamedClassImpl());
	element->setThisNamedClassPtr(element);
	return element;
}
std::shared_ptr<Student> UniModelExample_ecoreFactoryImpl::createStudent() const
{
	std::shared_ptr<StudentImpl> element(new StudentImpl());
	element->setThisStudentPtr(element);
	return element;
}
std::shared_ptr<University> UniModelExample_ecoreFactoryImpl::createUniversity() const
{
	std::shared_ptr<UniversityImpl> element(new UniversityImpl());
	element->setThisUniversityPtr(element);
	return element;
}

std::shared_ptr<UniModelExample_ecorePackage> UniModelExample_ecoreFactoryImpl::getUniModelExample_ecorePackage() const
{
	return UniModelExample_ecorePackage::eInstance();
}
