#include "uniModelExample_ecore/impl/UniModelExample_ecorePackageImpl.hpp"

// metametamodel factory
#include "ecore/EcoreFactory.hpp"

#include <cassert>

#include "abstractDataTypes/SubsetUnion.hpp"
//metametamodel classes
#include "ecore/EAttribute.hpp"
#include "ecore/EClass.hpp"
#include "ecore/EDataType.hpp"
#include "ecore/EOperation.hpp"
#include "ecore/EReference.hpp"

//depending model packages
#include "ecore/EcorePackage.hpp"

using namespace uniModelExample_ecore;

void UniModelExample_ecorePackageImpl::createPackageContents(std::shared_ptr<ecore::EPackage> package)
{
	if (isCreated) 
	{
		return;
	}
	isCreated = true;

	std::shared_ptr<ecore::EcoreFactory> factory = ecore::EcoreFactory::eInstance();

	createExamContent(package, factory);
	createFacultyContent(package, factory);
	createLectureContent(package, factory);
	createNamedClassContent(package, factory);
	createStudentContent(package, factory);
	createUniversityContent(package, factory);

	createPackageEDataTypes(package, factory);
}

void UniModelExample_ecorePackageImpl::createExamContent(std::shared_ptr<ecore::EPackage> package, std::shared_ptr<ecore::EcoreFactory> factory)
{
	m_exam_EClass = factory->createEClass_in_EPackage(package, EXAM_ECLASS);
	m_exam_EAttribute_score = factory->createEAttribute_in_EContainingClass(m_exam_EClass, EXAM_EATTRIBUTE_SCORE);
	
	m_exam_EReference_lecture = factory->createEReference_in_EContainingClass(m_exam_EClass, EXAM_EREFERENCE_LECTURE);
	
	
}

void UniModelExample_ecorePackageImpl::createFacultyContent(std::shared_ptr<ecore::EPackage> package, std::shared_ptr<ecore::EcoreFactory> factory)
{
	m_faculty_EClass = factory->createEClass_in_EPackage(package, FACULTY_ECLASS);
	
	m_faculty_EReference_lectures = factory->createEReference_in_EContainingClass(m_faculty_EClass, FACULTY_EREFERENCE_LECTURES);
	m_faculty_EReference_students = factory->createEReference_in_EContainingClass(m_faculty_EClass, FACULTY_EREFERENCE_STUDENTS);
	m_faculty_EReference_university = factory->createEReference_in_EContainingClass(m_faculty_EClass, FACULTY_EREFERENCE_UNIVERSITY);
	
	
}

void UniModelExample_ecorePackageImpl::createLectureContent(std::shared_ptr<ecore::EPackage> package, std::shared_ptr<ecore::EcoreFactory> factory)
{
	m_lecture_EClass = factory->createEClass_in_EPackage(package, LECTURE_ECLASS);
	
	
	
}

void UniModelExample_ecorePackageImpl::createNamedClassContent(std::shared_ptr<ecore::EPackage> package, std::shared_ptr<ecore::EcoreFactory> factory)
{
	m_namedClass_EClass = factory->createEClass_in_EPackage(package, NAMEDCLASS_ECLASS);
	m_namedClass_EAttribute_name = factory->createEAttribute_in_EContainingClass(m_namedClass_EClass, NAMEDCLASS_EATTRIBUTE_NAME);
	
	
	
}

void UniModelExample_ecorePackageImpl::createStudentContent(std::shared_ptr<ecore::EPackage> package, std::shared_ptr<ecore::EcoreFactory> factory)
{
	m_student_EClass = factory->createEClass_in_EPackage(package, STUDENT_ECLASS);
	m_student_EAttribute_matNum = factory->createEAttribute_in_EContainingClass(m_student_EClass, STUDENT_EATTRIBUTE_MATNUM);
	
	m_student_EReference_exams = factory->createEReference_in_EContainingClass(m_student_EClass, STUDENT_EREFERENCE_EXAMS);
	m_student_EReference_lectures = factory->createEReference_in_EContainingClass(m_student_EClass, STUDENT_EREFERENCE_LECTURES);
	
	m_student_EOperation_getAvgGrade = factory->createEOperation_in_EContainingClass(m_student_EClass, STUDENT_EOPERATION_GETAVGGRADE);
	
}

void UniModelExample_ecorePackageImpl::createUniversityContent(std::shared_ptr<ecore::EPackage> package, std::shared_ptr<ecore::EcoreFactory> factory)
{
	m_university_EClass = factory->createEClass_in_EPackage(package, UNIVERSITY_ECLASS);
	
	m_university_EReference_faculties = factory->createEReference_in_EContainingClass(m_university_EClass, UNIVERSITY_EREFERENCE_FACULTIES);
	
	m_university_EOperation_printUniversity = factory->createEOperation_in_EContainingClass(m_university_EClass, UNIVERSITY_EOPERATION_PRINTUNIVERSITY);
	
}

void UniModelExample_ecorePackageImpl::createPackageEDataTypes(std::shared_ptr<ecore::EPackage> package, std::shared_ptr<ecore::EcoreFactory> factory)
{
	
}
