#include "uniModelExample_ecore/impl/UniModelExample_ecorePackageImpl.hpp"

#include <cassert>

#include "abstractDataTypes/SubsetUnion.hpp"
//metametamodel classes
#include "ecore/EAttribute.hpp"
#include "ecore/EClass.hpp"
#include "ecore/EDataType.hpp"
#include "ecore/EOperation.hpp"
#include "ecore/EReference.hpp"

// metametamodel factory
#include "ecore/EcoreFactory.hpp"

//depending model packages
#include "ecore/EcorePackage.hpp"

using namespace uniModelExample_ecore;

void UniModelExample_ecorePackageImpl::initializePackageContents()
{
	if (isInitialized)
	{
		return;
	}
	isInitialized = true;

	// Initialize package
	setName(eNAME);
	setNsPrefix(eNS_PREFIX);
	setNsURI(eNS_URI);
	
	// Add supertypes to classes
	m_faculty_EClass->getESuperTypes()->push_back(getNamedClass_EClass());
	m_lecture_EClass->getESuperTypes()->push_back(getNamedClass_EClass());
	m_student_EClass->getESuperTypes()->push_back(getNamedClass_EClass());
	

 	// Initialize classes and features; add operations and parameters
	initializeExamContent();
	initializeFacultyContent();
	initializeLectureContent();
	initializeNamedClassContent();
	initializeStudentContent();
	initializeUniversityContent();

	initializePackageEDataTypes();
}

void UniModelExample_ecorePackageImpl::initializeExamContent()
{
	m_exam_EClass->setName("Exam");
	m_exam_EClass->setAbstract(false);
	m_exam_EClass->setInterface(false);
	
	m_exam_EAttribute_score = getExam_EAttribute_score();
	m_exam_EAttribute_score->setName("score");
	m_exam_EAttribute_score->setEType(ecore::EcorePackage::eInstance()->getEFloat_EDataType());
	m_exam_EAttribute_score->setLowerBound(0);
	m_exam_EAttribute_score->setUpperBound(1);
	m_exam_EAttribute_score->setTransient(false);
	m_exam_EAttribute_score->setVolatile(false);
	m_exam_EAttribute_score->setChangeable(true);
	m_exam_EAttribute_score->setUnsettable(false);
	m_exam_EAttribute_score->setUnique(true);
	m_exam_EAttribute_score->setDerived(false);
	m_exam_EAttribute_score->setOrdered(true);
	m_exam_EAttribute_score->setID(false);
	{
		std::string defaultValue = "";
		if (!defaultValue.empty())
		{
		   m_exam_EAttribute_score->setDefaultValueLiteral(defaultValue);
		}
	}
	
	m_exam_EReference_lecture->setName("lecture");
	m_exam_EReference_lecture->setEType(getLecture_EClass());
	m_exam_EReference_lecture->setLowerBound(0);
	m_exam_EReference_lecture->setUpperBound(1);
	m_exam_EReference_lecture->setTransient(false);
	m_exam_EReference_lecture->setVolatile(false);
	m_exam_EReference_lecture->setChangeable(true);
	m_exam_EReference_lecture->setUnsettable(false);
	m_exam_EReference_lecture->setUnique(true);
	m_exam_EReference_lecture->setDerived(false);
	m_exam_EReference_lecture->setOrdered(true);
	m_exam_EReference_lecture->setContainment(false);
	m_exam_EReference_lecture->setResolveProxies(true);
	{
		std::string defaultValue = "";
		if (!defaultValue.empty())
		{
			m_exam_EReference_lecture->setDefaultValueLiteral(defaultValue);
		}
		std::shared_ptr<ecore::EReference>  otherEnd = nullptr;
		if (otherEnd != nullptr)
	    {
	   		m_exam_EReference_lecture->setEOpposite(otherEnd);
	    }
	}
	
	
}

void UniModelExample_ecorePackageImpl::initializeFacultyContent()
{
	m_faculty_EClass->setName("Faculty");
	m_faculty_EClass->setAbstract(false);
	m_faculty_EClass->setInterface(false);
	
	
	m_faculty_EReference_lectures->setName("lectures");
	m_faculty_EReference_lectures->setEType(getLecture_EClass());
	m_faculty_EReference_lectures->setLowerBound(0);
	m_faculty_EReference_lectures->setUpperBound(-1);
	m_faculty_EReference_lectures->setTransient(false);
	m_faculty_EReference_lectures->setVolatile(false);
	m_faculty_EReference_lectures->setChangeable(true);
	m_faculty_EReference_lectures->setUnsettable(false);
	m_faculty_EReference_lectures->setUnique(true);
	m_faculty_EReference_lectures->setDerived(false);
	m_faculty_EReference_lectures->setOrdered(true);
	m_faculty_EReference_lectures->setContainment(true);
	m_faculty_EReference_lectures->setResolveProxies(true);
	{
		std::string defaultValue = "";
		if (!defaultValue.empty())
		{
			m_faculty_EReference_lectures->setDefaultValueLiteral(defaultValue);
		}
		std::shared_ptr<ecore::EReference>  otherEnd = nullptr;
		if (otherEnd != nullptr)
	    {
	   		m_faculty_EReference_lectures->setEOpposite(otherEnd);
	    }
	}
	m_faculty_EReference_students->setName("students");
	m_faculty_EReference_students->setEType(getStudent_EClass());
	m_faculty_EReference_students->setLowerBound(0);
	m_faculty_EReference_students->setUpperBound(-1);
	m_faculty_EReference_students->setTransient(false);
	m_faculty_EReference_students->setVolatile(false);
	m_faculty_EReference_students->setChangeable(true);
	m_faculty_EReference_students->setUnsettable(false);
	m_faculty_EReference_students->setUnique(true);
	m_faculty_EReference_students->setDerived(false);
	m_faculty_EReference_students->setOrdered(true);
	m_faculty_EReference_students->setContainment(false);
	m_faculty_EReference_students->setResolveProxies(true);
	{
		std::string defaultValue = "";
		if (!defaultValue.empty())
		{
			m_faculty_EReference_students->setDefaultValueLiteral(defaultValue);
		}
		std::shared_ptr<ecore::EReference>  otherEnd = nullptr;
		if (otherEnd != nullptr)
	    {
	   		m_faculty_EReference_students->setEOpposite(otherEnd);
	    }
	}
	m_faculty_EReference_university->setName("university");
	m_faculty_EReference_university->setEType(getUniversity_EClass());
	m_faculty_EReference_university->setLowerBound(0);
	m_faculty_EReference_university->setUpperBound(1);
	m_faculty_EReference_university->setTransient(false);
	m_faculty_EReference_university->setVolatile(false);
	m_faculty_EReference_university->setChangeable(true);
	m_faculty_EReference_university->setUnsettable(false);
	m_faculty_EReference_university->setUnique(true);
	m_faculty_EReference_university->setDerived(false);
	m_faculty_EReference_university->setOrdered(true);
	m_faculty_EReference_university->setContainment(false);
	m_faculty_EReference_university->setResolveProxies(true);
	{
		std::string defaultValue = "";
		if (!defaultValue.empty())
		{
			m_faculty_EReference_university->setDefaultValueLiteral(defaultValue);
		}
		std::shared_ptr<ecore::EReference>  otherEnd = getUniversity_EReference_faculties();
		if (otherEnd != nullptr)
	    {
	   		m_faculty_EReference_university->setEOpposite(otherEnd);
	    }
	}
	
	
}

void UniModelExample_ecorePackageImpl::initializeLectureContent()
{
	m_lecture_EClass->setName("Lecture");
	m_lecture_EClass->setAbstract(false);
	m_lecture_EClass->setInterface(false);
	
	
	
	
}

void UniModelExample_ecorePackageImpl::initializeNamedClassContent()
{
	m_namedClass_EClass->setName("NamedClass");
	m_namedClass_EClass->setAbstract(false);
	m_namedClass_EClass->setInterface(false);
	
	m_namedClass_EAttribute_name = getNamedClass_EAttribute_name();
	m_namedClass_EAttribute_name->setName("name");
	m_namedClass_EAttribute_name->setEType(ecore::EcorePackage::eInstance()->getEString_EDataType());
	m_namedClass_EAttribute_name->setLowerBound(0);
	m_namedClass_EAttribute_name->setUpperBound(1);
	m_namedClass_EAttribute_name->setTransient(false);
	m_namedClass_EAttribute_name->setVolatile(false);
	m_namedClass_EAttribute_name->setChangeable(true);
	m_namedClass_EAttribute_name->setUnsettable(false);
	m_namedClass_EAttribute_name->setUnique(true);
	m_namedClass_EAttribute_name->setDerived(false);
	m_namedClass_EAttribute_name->setOrdered(true);
	m_namedClass_EAttribute_name->setID(false);
	{
		std::string defaultValue = "";
		if (!defaultValue.empty())
		{
		   m_namedClass_EAttribute_name->setDefaultValueLiteral(defaultValue);
		}
	}
	
	
	
}

void UniModelExample_ecorePackageImpl::initializeStudentContent()
{
	m_student_EClass->setName("Student");
	m_student_EClass->setAbstract(false);
	m_student_EClass->setInterface(false);
	
	m_student_EAttribute_matNum = getStudent_EAttribute_matNum();
	m_student_EAttribute_matNum->setName("matNum");
	m_student_EAttribute_matNum->setEType(ecore::EcorePackage::eInstance()->getEInt_EDataType());
	m_student_EAttribute_matNum->setLowerBound(0);
	m_student_EAttribute_matNum->setUpperBound(1);
	m_student_EAttribute_matNum->setTransient(false);
	m_student_EAttribute_matNum->setVolatile(false);
	m_student_EAttribute_matNum->setChangeable(true);
	m_student_EAttribute_matNum->setUnsettable(false);
	m_student_EAttribute_matNum->setUnique(true);
	m_student_EAttribute_matNum->setDerived(false);
	m_student_EAttribute_matNum->setOrdered(true);
	m_student_EAttribute_matNum->setID(false);
	{
		std::string defaultValue = "";
		if (!defaultValue.empty())
		{
		   m_student_EAttribute_matNum->setDefaultValueLiteral(defaultValue);
		}
	}
	
	m_student_EReference_exams->setName("exams");
	m_student_EReference_exams->setEType(getExam_EClass());
	m_student_EReference_exams->setLowerBound(0);
	m_student_EReference_exams->setUpperBound(-1);
	m_student_EReference_exams->setTransient(false);
	m_student_EReference_exams->setVolatile(false);
	m_student_EReference_exams->setChangeable(true);
	m_student_EReference_exams->setUnsettable(false);
	m_student_EReference_exams->setUnique(true);
	m_student_EReference_exams->setDerived(false);
	m_student_EReference_exams->setOrdered(true);
	m_student_EReference_exams->setContainment(true);
	m_student_EReference_exams->setResolveProxies(true);
	{
		std::string defaultValue = "";
		if (!defaultValue.empty())
		{
			m_student_EReference_exams->setDefaultValueLiteral(defaultValue);
		}
		std::shared_ptr<ecore::EReference>  otherEnd = nullptr;
		if (otherEnd != nullptr)
	    {
	   		m_student_EReference_exams->setEOpposite(otherEnd);
	    }
	}
	m_student_EReference_lectures->setName("lectures");
	m_student_EReference_lectures->setEType(getLecture_EClass());
	m_student_EReference_lectures->setLowerBound(0);
	m_student_EReference_lectures->setUpperBound(-1);
	m_student_EReference_lectures->setTransient(false);
	m_student_EReference_lectures->setVolatile(false);
	m_student_EReference_lectures->setChangeable(true);
	m_student_EReference_lectures->setUnsettable(false);
	m_student_EReference_lectures->setUnique(true);
	m_student_EReference_lectures->setDerived(false);
	m_student_EReference_lectures->setOrdered(true);
	m_student_EReference_lectures->setContainment(false);
	m_student_EReference_lectures->setResolveProxies(true);
	{
		std::string defaultValue = "";
		if (!defaultValue.empty())
		{
			m_student_EReference_lectures->setDefaultValueLiteral(defaultValue);
		}
		std::shared_ptr<ecore::EReference>  otherEnd = nullptr;
		if (otherEnd != nullptr)
	    {
	   		m_student_EReference_lectures->setEOpposite(otherEnd);
	    }
	}
	
	m_student_EOperation_getAvgGrade->setEType(ecore::EcorePackage::eInstance()->getEFloat_EDataType());
	m_student_EOperation_getAvgGrade->setName("getAvgGrade");
	m_student_EOperation_getAvgGrade->setLowerBound(0);
	m_student_EOperation_getAvgGrade->setUpperBound(1);
	m_student_EOperation_getAvgGrade->setUnique(true);
	m_student_EOperation_getAvgGrade->setOrdered(true);
	
	
	
}

void UniModelExample_ecorePackageImpl::initializeUniversityContent()
{
	m_university_EClass->setName("University");
	m_university_EClass->setAbstract(false);
	m_university_EClass->setInterface(false);
	
	
	m_university_EReference_faculties->setName("faculties");
	m_university_EReference_faculties->setEType(getFaculty_EClass());
	m_university_EReference_faculties->setLowerBound(0);
	m_university_EReference_faculties->setUpperBound(-1);
	m_university_EReference_faculties->setTransient(false);
	m_university_EReference_faculties->setVolatile(false);
	m_university_EReference_faculties->setChangeable(true);
	m_university_EReference_faculties->setUnsettable(false);
	m_university_EReference_faculties->setUnique(true);
	m_university_EReference_faculties->setDerived(false);
	m_university_EReference_faculties->setOrdered(true);
	m_university_EReference_faculties->setContainment(true);
	m_university_EReference_faculties->setResolveProxies(true);
	{
		std::string defaultValue = "";
		if (!defaultValue.empty())
		{
			m_university_EReference_faculties->setDefaultValueLiteral(defaultValue);
		}
		std::shared_ptr<ecore::EReference>  otherEnd = getFaculty_EReference_university();
		if (otherEnd != nullptr)
	    {
	   		m_university_EReference_faculties->setEOpposite(otherEnd);
	    }
	}
	
	m_university_EOperation_printUniversity->setEType(nullptr);
	m_university_EOperation_printUniversity->setName("printUniversity");
	m_university_EOperation_printUniversity->setLowerBound(0);
	m_university_EOperation_printUniversity->setUpperBound(1);
	m_university_EOperation_printUniversity->setUnique(true);
	m_university_EOperation_printUniversity->setOrdered(true);
	
	
	
}

void UniModelExample_ecorePackageImpl::initializePackageEDataTypes()
{
	
}
