#include "uniModelExample_ecore/impl/UniversityImpl.hpp"

#ifdef NDEBUG
	#define DEBUG_MESSAGE(a) /**/
#else
	#define DEBUG_MESSAGE(a) a
#endif

#ifdef ACTIVITY_DEBUG_ON
    #define ACT_DEBUG(a) a
#else
    #define ACT_DEBUG(a) /**/
#endif

//#include "util/ProfileCallCount.hpp"

#include <cassert>
#include <iostream>
#include <sstream>

#include "abstractDataTypes/Bag.hpp"

#include "abstractDataTypes/SubsetUnion.hpp"
#include "ecore/EAnnotation.hpp"
#include "ecore/EClass.hpp"
#include "uniModelExample_ecore/impl/UniModelExample_ecorePackageImpl.hpp"
#include "uniModelExample_ecore/Student.hpp"
#include "uniModelExample_ecore/Lecture.hpp"
#include <iomanip>

//Forward declaration includes
#include "persistence/interfaces/XLoadHandler.hpp" // used for Persistence
#include "persistence/interfaces/XSaveHandler.hpp" // used for Persistence
#include "uniModelExample_ecore/UniModelExample_ecoreFactory.hpp"
#include "uniModelExample_ecore/UniModelExample_ecorePackage.hpp"
#include <exception> // used in Persistence

#include "uniModelExample_ecore/Faculty.hpp"

#include "ecore/EcorePackage.hpp"
#include "ecore/EcoreFactory.hpp"
#include "uniModelExample_ecore/UniModelExample_ecorePackage.hpp"
#include "uniModelExample_ecore/UniModelExample_ecoreFactory.hpp"
#include "ecore/EAttribute.hpp"
#include "ecore/EStructuralFeature.hpp"

using namespace uniModelExample_ecore;

//*********************************
// Constructor / Destructor
//*********************************
UniversityImpl::UniversityImpl()
{
	//*********************************
	// Attribute Members
	//*********************************

	//*********************************
	// Reference Members
	//*********************************
	//References
		m_faculties.reset(new Bag<uniModelExample_ecore::Faculty>());
	
	

	//Init references
	
	
}

UniversityImpl::~UniversityImpl()
{
#ifdef SHOW_DELETION
	std::cout << "-------------------------------------------------------------------------------------------------\r\ndelete University "<< this << "\r\n------------------------------------------------------------------------ " << std::endl;
#endif
}




UniversityImpl::UniversityImpl(const UniversityImpl & obj):UniversityImpl()
{
	//create copy of all Attributes
	#ifdef SHOW_COPIES
	std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\r\ncopy University "<< this << "\r\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ " << std::endl;
	#endif

	//copy references with no containment (soft copy)
	

	//Clone references with containment (deep copy)

	std::shared_ptr<Bag<uniModelExample_ecore::Faculty>> _facultiesList = obj.getFaculties();
	for(std::shared_ptr<uniModelExample_ecore::Faculty> _faculties : *_facultiesList)
	{
		this->getFaculties()->add(std::shared_ptr<uniModelExample_ecore::Faculty>(std::dynamic_pointer_cast<uniModelExample_ecore::Faculty>(_faculties->copy())));
	}
	#ifdef SHOW_SUBSET_UNION
		std::cout << "Copying the Subset: " << "m_faculties" << std::endl;
	#endif

	
	
}

std::shared_ptr<ecore::EObject>  UniversityImpl::copy() const
{
	std::shared_ptr<UniversityImpl> element(new UniversityImpl(*this));
	element->setThisUniversityPtr(element);
	return element;
}

std::shared_ptr<ecore::EClass> UniversityImpl::eStaticClass() const
{
	return UniModelExample_ecorePackageImpl::eInstance()->getUniversity_EClass();
}

//*********************************
// Attribute Setter Getter
//*********************************

//*********************************
// Operations
//*********************************
void UniversityImpl::printUniversity()
{
	//ADD_COUNT(__PRETTY_FUNCTION__)
	//generated from body annotation
	//Get container of all faculties of university
std::shared_ptr<Bag<uniModelExample_ecore::Faculty>> faculties = this->getFaculties();

//Loop over all faculties
for(Bag<uniModelExample_ecore::Faculty>::const_iterator it_fac = faculties->begin(); it_fac != faculties->end(); it_fac++)
{
	//Get container of all students in current faculty
	std::shared_ptr<Bag<uniModelExample_ecore::Student>> students = (*it_fac)->getStudents();

	//Loop over all students 
	for(Bag<uniModelExample_ecore::Student>::const_iterator it_stud = students->begin(); it_stud != students->end(); it_stud++)
	{
		std::cout<<std::endl<<"Student " << (*it_stud)->getName()<<" of faculty " <<(*it_fac)->getName()<<std::endl;	
		std::cout<<"He/She took the following courses:"<<std::endl;
		
		//Get container of all lectures of current student
		std::shared_ptr<Bag<uniModelExample_ecore::Lecture>> lectures = (*it_stud)->getLectures();

		//Loop over all lectures
		for(Bag<uniModelExample_ecore::Lecture>::const_iterator it_lec = lectures->begin(); it_lec != lectures->end(); it_lec++){
			std::cout<<"\t- "<<(*it_lec)->getName()<<std::endl;
		}

	std::cout<<"His/Her average grade is: "<<std::setprecision(2)<<(*it_stud)->getAvgGrade()<<std::endl;
	}

}
	//end of body
}

//*********************************
// References
//*********************************
std::shared_ptr<Bag<uniModelExample_ecore::Faculty>> UniversityImpl::getFaculties() const
{

    return m_faculties;
}


//*********************************
// Union Getter
//*********************************


std::shared_ptr<University> UniversityImpl::getThisUniversityPtr() const
{
	return m_thisUniversityPtr.lock();
}
void UniversityImpl::setThisUniversityPtr(std::weak_ptr<University> thisUniversityPtr)
{
	m_thisUniversityPtr = thisUniversityPtr;
}
std::shared_ptr<ecore::EObject> UniversityImpl::eContainer() const
{
	return nullptr;
}

//*********************************
// Structural Feature Getter/Setter
//*********************************
Any UniversityImpl::eGet(int featureID, bool resolve, bool coreType) const
{
	switch(featureID)
	{
		case UniModelExample_ecorePackage::UNIVERSITY_EREFERENCE_FACULTIES:
			return eAny(getFaculties()); //60
	}
	return ecore::EObjectImpl::eGet(featureID, resolve, coreType);
}
bool UniversityImpl::internalEIsSet(int featureID) const
{
	switch(featureID)
	{
		case UniModelExample_ecorePackage::UNIVERSITY_EREFERENCE_FACULTIES:
			return getFaculties() != nullptr; //60
	}
	return ecore::EObjectImpl::internalEIsSet(featureID);
}
bool UniversityImpl::eSet(int featureID, Any newValue)
{
	switch(featureID)
	{
	}

	return ecore::EObjectImpl::eSet(featureID, newValue);
}

//*********************************
// Persistence Functions
//*********************************
void UniversityImpl::load(std::shared_ptr<persistence::interfaces::XLoadHandler> loadHandler)
{
	std::map<std::string, std::string> attr_list = loadHandler->getAttributeList();
	loadAttributes(loadHandler, attr_list);

	//
	// Create new objects (from references (containment == true))
	//
	// get UniModelExample_ecoreFactory
	std::shared_ptr<uniModelExample_ecore::UniModelExample_ecoreFactory> modelFactory = uniModelExample_ecore::UniModelExample_ecoreFactory::eInstance();
	int numNodes = loadHandler->getNumOfChildNodes();
	for(int ii = 0; ii < numNodes; ii++)
	{
		loadNode(loadHandler->getNextNodeName(), loadHandler, modelFactory);
	}
}		

void UniversityImpl::loadAttributes(std::shared_ptr<persistence::interfaces::XLoadHandler> loadHandler, std::map<std::string, std::string> attr_list)
{

	ecore::EObjectImpl::loadAttributes(loadHandler, attr_list);
}

void UniversityImpl::loadNode(std::string nodeName, std::shared_ptr<persistence::interfaces::XLoadHandler> loadHandler, std::shared_ptr<uniModelExample_ecore::UniModelExample_ecoreFactory> modelFactory)
{

	try
	{
		if ( nodeName.compare("faculties") == 0 )
		{
  			std::string typeName = loadHandler->getCurrentXSITypeName();
			if (typeName.empty())
			{
				typeName = "Faculty";
			}
			std::shared_ptr<ecore::EObject> faculties = modelFactory->create(typeName, loadHandler->getCurrentObject(), UniModelExample_ecorePackage::FACULTY_EREFERENCE_UNIVERSITY);
			if (faculties != nullptr)
			{
				loadHandler->handleChild(faculties);
			}
			return;
		}
	}
	catch (std::exception& e)
	{
		std::cout << "| ERROR    | " << e.what() << std::endl;
	}
	catch (...) 
	{
		std::cout << "| ERROR    | " <<  "Exception occurred" << std::endl;
	}

	ecore::EObjectImpl::loadNode(nodeName, loadHandler, ecore::EcoreFactory::eInstance());
}

void UniversityImpl::resolveReferences(const int featureID, std::list<std::shared_ptr<ecore::EObject> > references)
{
	ecore::EObjectImpl::resolveReferences(featureID, references);
}

void UniversityImpl::save(std::shared_ptr<persistence::interfaces::XSaveHandler> saveHandler) const
{
	saveContent(saveHandler);

	
	ecore::EObjectImpl::saveContent(saveHandler);
	
}

void UniversityImpl::saveContent(std::shared_ptr<persistence::interfaces::XSaveHandler> saveHandler) const
{
	try
	{
		std::shared_ptr<uniModelExample_ecore::UniModelExample_ecorePackage> package = uniModelExample_ecore::UniModelExample_ecorePackage::eInstance();

	


		//
		// Add new tags (from references)
		//
		std::shared_ptr<ecore::EClass> metaClass = this->eClass();
		// Save 'faculties'
		std::shared_ptr<Bag<uniModelExample_ecore::Faculty>> list_faculties = this->getFaculties();
		for (std::shared_ptr<uniModelExample_ecore::Faculty> faculties : *list_faculties) 
		{
			saveHandler->addReference(faculties, "faculties", faculties->eClass() != package->getFaculty_EClass());
		}
	}
	catch (std::exception& e)
	{
		std::cout << "| ERROR    | " << e.what() << std::endl;
	}
}

